@@ -148,7 +148,7 @@ formatting = {
             'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
-            'unfocused': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
+            'unfocused': 'label + readOnly + textRole + currentLineText + anyTextSelection + ' + MNEMONIC,
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
@@ -325,6 +325,9 @@ formatting = {
                               + (expandableState and (expandableState + numberOfChildren))\
                               + required)'
             },
+        pyatspi.ROLE_TABLE_ROW: {
+            'focused': 'expandableState',
+            },
         pyatspi.ROLE_TEAROFF_MENU_ITEM: {
             'focused': '[]',
             'unfocused': 'labelOrName + allTextSelection + roleName + availability '
