@@ -91,53 +91,6 @@ class Utilities:
     #                                                                       #
     #########################################################################
 
-    @staticmethod
-    def __hasLabelForRelation(label):
-        """Check if label has a LABEL_FOR relation
-
-        Arguments:
-        - label: the label in question
-
-        Returns TRUE if label has a LABEL_FOR relation.
-        """
-        if (not label) or (label.getRole() != pyatspi.ROLE_LABEL):
-            return False
-
-        relations = label.getRelationSet()
-
-        for relation in relations:
-            if relation.getRelationType() == pyatspi.RELATION_LABEL_FOR:
-                return True
-
-        return False
-
-    @staticmethod
-    def __isLabeling(label, obj):
-        """Check if label is connected via  LABEL_FOR relation with object
-
-        Arguments:
-        - obj: the object in question
-        - labeled: the label in question
-
-        Returns TRUE if label has a relation LABEL_FOR for object.
-        """
-
-        if not obj or not label or label.getRole() != pyatspi.ROLE_LABEL:
-            return False
-
-        relations = label.getRelationSet()
-        if not relations:
-            return False
-
-        for relation in relations:
-            if relation.getRelationType() == pyatspi.RELATION_LABEL_FOR:
-                for i in range(0, relation.getNTargets()):
-                    target = relation.getTarget(i)
-                    if target == obj:
-                        return True
-
-        return False
-
     @staticmethod
     def activeWindow():
         """Traverses the list of known apps looking for one who has an
@@ -159,42 +112,6 @@ class Utilities:
 
         return None
 
-    @staticmethod
-    def allDescendants(root, onlyShowing=True):
-        """Returns a list of all objects under the given root.  Objects
-        are returned in no particular order - this function does a simple
-        tree traversal, ignoring the children of objects which report the
-        MANAGES_DESCENDANTS state.
-
-        Arguments:
-        - root: the Accessible object to traverse
-        - onlyShowing: examine only those objects that are SHOWING
-
-        Returns: a list of all objects under the specified object
-        """
-
-        if root and root.childCount <= 0:
-            return []
-
-        objlist = []
-        for i, child in enumerate(root):
-            debug.println(debug.LEVEL_FINEST,
-                          "Script.allDescendants looking at child %d" % i)
-            try:
-                state = child.getState()
-            except:
-                debug.println(debug.LEVEL_FINEST, "Error getting state")
-                continue
-
-            if not onlyShowing \
-               or (onlyShowing and state.contains(pyatspi.STATE_SHOWING)):
-                objlist.append(child)
-                if not state.contains(pyatspi.STATE_MANAGES_DESCENDANTS) \
-                   and child.childCount > 0:
-                    objlist.extend(Utilities.allDescendants(child, onlyShowing))
-
-        return objlist
-
     @staticmethod
     def ancestorWithRole(obj, ancestorRoles, stopRoles):
         """Returns the object of the specified roles which contains the
@@ -434,22 +351,6 @@ class Utilities:
 
         return defaultButton
 
-    @staticmethod
-    def descendantsWithRole(root, role, onlyShowing=True):
-        """Returns a list of all objects of a specific role beneath the
-        given root.
-
-        Arguments:
-        - root: the Accessible object to traverse
-        - role: the string describing the Accessible role of the object
-        - onlyShowing: examine only those objects that are SHOWING
-
-        Returns a list of descendants of the root that are of the given role.
-        """
-
-        allObjs = Utilities.allDescendants(root, onlyShowing)
-        return [o for o in allObjs if o.getRole() == role]
-
     def displayedLabel(self, obj):
         """If there is an object labelling the given object, return the
         text being displayed for the object labelling this object.
@@ -1445,7 +1346,8 @@ class Utilities:
         Returns a list of unrelated labels under the given root.
         """
 
-        allLabels = self.descendantsWithRole(root, pyatspi.ROLE_LABEL, onlyShowing)
+        hasRole = lambda x: x and x.getRole() == pyatspi.ROLE_LABEL
+        allLabels = pyatspi.findAllDescendants(root, hasRole)
         try:
             labels = [x for x in allLabels if not x.getRelationSet()]
             labels = [x for x in labels if x.parent and x.name != x.parent.name]
@@ -1660,7 +1562,8 @@ class Utilities:
         try:
             hyperlink = obj.queryHyperlink()
         except NotImplementedError:
-            pass
+            msg = "INFO: %s does not implement the hyperlink interface" % obj
+            debug.println(debug.LEVEL_INFO, msg)
         else:
             # We need to make sure that this is an embedded object in
             # some accessible text (as opposed to an imagemap link).
@@ -1669,7 +1572,11 @@ class Utilities:
                 obj.parent.queryText()
                 offset = hyperlink.startIndex
             except:
-                pass
+                msg = "ERROR: Exception getting startIndex for %s in parent %s" % (obj, obj.parent)
+                debug.println(debug.LEVEL_INFO, msg)
+            else:
+                msg = "INFO: startIndex of %s is %i" % (obj, offset)
+                debug.println(debug.LEVEL_INFO, msg)
 
         return offset
 
@@ -1731,71 +1638,6 @@ class Utilities:
 
         return string
 
-    def hasTextSelections(self, obj):
-        """Return an indication of whether this object has selected text.
-        Note that it's possible that this object has no selection, but is part
-        of a selected text area. Because of this, we need to check the
-        objects on either side to see if they are none zero length and
-        have text selections.
-
-        Arguments:
-        - obj: the text object to start checking for selected text.
-
-        Returns: an indication of whether this object has selected text,
-        or adjacent text objects have selected text.
-        """
-
-        currentSelected = False
-        otherSelected = False
-        text = obj.queryText()
-        nSelections = text.getNSelections()
-        if nSelections:
-            currentSelected = True
-        else:
-            otherSelected = False
-            text = obj.queryText()
-            displayedText = text.getText(0, text.characterCount)
-            if (text.caretOffset == 0) or len(displayedText) == 0:
-                current = obj
-                morePossibleSelections = True
-                while morePossibleSelections:
-                    morePossibleSelections = False
-                    for relation in current.getRelationSet():
-                        if relation.getRelationType() == \
-                               pyatspi.RELATION_FLOWS_FROM:
-                            prevObj = relation.getTarget(0)
-                            prevObjText = prevObj.queryText()
-                            if prevObjText.getNSelections() > 0:
-                                otherSelected = True
-                            else:
-                                displayedText = prevObjText.getText(0,
-                                    prevObjText.characterCount)
-                                if len(displayedText) == 0:
-                                    current = prevObj
-                                    morePossibleSelections = True
-                            break
-
-                current = obj
-                morePossibleSelections = True
-                while morePossibleSelections:
-                    morePossibleSelections = False
-                    for relation in current.getRelationSet():
-                        if relation.getRelationType() == \
-                               pyatspi.RELATION_FLOWS_TO:
-                            nextObj = relation.getTarget(0)
-                            nextObjText = nextObj.queryText()
-                            if nextObjText.getNSelections() > 0:
-                                otherSelected = True
-                            else:
-                                displayedText = nextObjText.getText(0,
-                                    nextObjText.characterCount)
-                                if len(displayedText) == 0:
-                                    current = nextObj
-                                    morePossibleSelections = True
-                            break
-
-        return [currentSelected, otherSelected]
-
     def isWordMisspelled(self, obj, offset):
         """Identifies if the current word is flagged as misspelled by the
         application. Different applications and toolkits flag misspelled
@@ -2570,14 +2412,22 @@ class Utilities:
         role = obj.getRole()
         if role == pyatspi.ROLE_MENU and not children:
             pred = lambda x: x and x.getState().contains(pyatspi.STATE_SELECTED)
-            children = pyatspi.findAllDescendants(obj, pred)
+            try:
+                children = pyatspi.findAllDescendants(obj, pred)
+            except:
+                msg = "ERROR: Exception calling findAllDescendants on %s" % obj
+                debug.println(debug.LEVEL_INFO, msg)
 
         if role == pyatspi.ROLE_COMBO_BOX \
            and children and children[0].getRole() == pyatspi.ROLE_MENU:
             children = self.selectedChildren(children[0])
             if not children and obj.name:
                 pred = lambda x: x and x.name == obj.name
-                children = pyatspi.findAllDescendants(obj, pred)
+                try:
+                    children = pyatspi.findAllDescendants(obj, pred)
+                except:
+                    msg = "ERROR: Exception calling findAllDescendants on %s" % obj
+                    debug.println(debug.LEVEL_INFO, msg)
 
         return children
 
