@@ -69,7 +69,11 @@ class Utilities(script_utilities.Utilities):
         any text being shown.
         """
 
-        role = obj.getRole()
+        try:
+            role = obj.getRole()
+        except:
+            return ""
+
         if role == pyatspi.ROLE_PUSH_BUTTON and obj.name:
             return obj.name
 
@@ -79,7 +83,10 @@ class Utilities(script_utilities.Utilities):
             if text.strip():
                 return text
 
-        text = script_utilities.Utilities.displayedText(self, obj)
+        try:
+            text = script_utilities.Utilities.displayedText(self, obj)
+        except:
+            return ""
 
         # TODO - JD: This is needed because the default behavior is to fall
         # back on the name, which is bogus. Once that has been fixed, this
@@ -512,15 +519,7 @@ class Utilities(script_utilities.Utilities):
         """Attempts to locate the Impress drawing view, which is the
         area in which slide editing occurs."""
 
-        # TODO - JD: We should probably add this to the generatorCache.
-        #
-        docFrames = self.descendantsWithRole(
-            self.topLevelObject(obj), pyatspi.ROLE_DOCUMENT_FRAME)
-        docFrame = [o for o in docFrames if ":" in o.name and "/" in o.name]
-        if docFrame:
-            return docFrame[0]
-
-        return None
+        return pyatspi.findDescendant(self.topLevelObject(obj), self.isDrawingView)
 
     def isDrawingView(self, obj):
         """Returns True if obj is the Impress Drawing View."""
@@ -538,7 +537,8 @@ class Utilities(script_utilities.Utilities):
         #
         if obj:
             topLevel = self.topLevelObject(obj)
-            if topLevel and topLevel.name.endswith("Impress"):
+            if topLevel and not self.isZombie(topLevel) \
+               and topLevel.name.endswith("Impress"):
                 return True
 
         return False
@@ -556,12 +556,14 @@ class Utilities(script_utilities.Utilities):
         if not parent:
             return None, None
 
-        panes = self.descendantsWithRole(parent, pyatspi.ROLE_SPLIT_PANE)
+        hasRole = lambda x: x and x.getRole() == pyatspi.ROLE_SPLIT_PANE
+        panes = pyatspi.findAllDescendants(parent, hasRole)
         if not panes:
             return None, None
 
         slidePane = taskPane = None
-        if self.descendantsWithRole(panes[0], pyatspi.ROLE_DOCUMENT_FRAME):
+        hasRole = lambda x: x and x.getRole() == pyatspi.ROLE_DOCUMENT_FRAME
+        if pyatspi.findAllDescendants(panes[0], hasRole):
             slidePane = panes[0]
             if len(panes) == 2:
                 taskPane = panes[1]
