@@ -71,42 +71,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         result = []
         acss = self.voice(speech_generator.DEFAULT)
         role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_COMBO_BOX:
-            # With Gecko, a combo box has a menu as a child.  The text being
-            # displayed for the combo box can be obtained via the selected
-            # menu item.
-            #
-            menu = None
-            for child in obj:
-                if child.getRole() == pyatspi.ROLE_MENU:
-                    menu = child
-                    break
-            if menu:
-                child = None
-                try:
-                    # This should work...
-                    #
-                    child = menu.querySelection().getSelectedChild(0)
-                    if not child:
-                        # It's probably a Gtk combo box.
-                        #
-                        result = speech_generator.SpeechGenerator.\
-                            _generateDisplayedText(self, obj, **args)
-                except:
-                    # But just in case, we'll fall back on this.
-                    # [[[TODO - JD: Will we ever have a case where the first
-                    # fails, but this will succeed???]]]
-                    #
-                    for item in menu:
-                        if item.getState().contains(pyatspi.STATE_SELECTED):
-                            child = item
-                            break
-                if child and child.name:
-                    result.append(child.name)
-                    result.extend(acss)
+        if role == pyatspi.ROLE_DOCUMENT_FRAME \
+           and obj.getState().contains(pyatspi.STATE_EDITABLE):
+            return []
 
-        else:
-            result.extend(speech_generator.SpeechGenerator._generateName(
+        result.extend(speech_generator.SpeechGenerator._generateName(
                               self, obj, **args))
         if not result and role == pyatspi.ROLE_LIST_ITEM:
             result.append(self._script.utilities.expandEOCs(obj))
@@ -148,16 +117,18 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             self, obj, **args)
 
     def _generateLabel(self, obj, **args):
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if isinstance(start, int) and isinstance(end, int) \
+           and not self._script.utilities.justEnteredObject(obj, start, end):
+            return []
+
         acss = self.voice(speech_generator.DEFAULT)
         result = speech_generator.SpeechGenerator._generateLabel(self,
                                                                  obj,
                                                                  **args)
 
         if self._script.utilities.shouldInferLabelFor(obj):
-            start = args.get('startOffset')
-            if isinstance(start, int) and start > 0:
-                return []
-
             label, objects = self._script.labelInference.infer(obj, False)
             if label:
                 result.append(label)
@@ -180,6 +151,12 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if isinstance(start, int) and isinstance(end, int) \
+           and not self._script.utilities.justEnteredObject(obj, start, end):
+            return []
+
         result = speech_generator.SpeechGenerator._generateLabelOrName(
             self, obj, **args)
 
@@ -195,19 +172,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         role = args.get('role', obj.getRole())
         force = args.get('force', False)
         start = args.get('startOffset')
-        if role == pyatspi.ROLE_ENTRY and isinstance(start, int) and start > 0:
+        end = args.get('endOffset')
+        if isinstance(start, int) and isinstance(end, int) \
+           and not self._script.utilities.justEnteredObject(obj, start, end) \
+           and not self._script.utilities.isTextBlockElement(obj) \
+           and not self._script.utilities.isLink(obj):
+            return []
+        if role == pyatspi.ROLE_DOCUMENT_FRAME \
+           and obj.getState().contains(pyatspi.STATE_EDITABLE):
             return []
-
-        # Saying "menu item" for a combo box can confuse users. Therefore,
-        # speak the combo box role instead.  Also, only do it if the menu
-        # item is not focused (if the menu item is focused, it means we're
-        # navigating in the combo box)
-        #
-        if not obj.getState().contains(pyatspi.STATE_FOCUSED):
-            comboBox = self._script.utilities.ancestorWithRole(
-                obj, [pyatspi.ROLE_COMBO_BOX], [pyatspi.ROLE_DOCUMENT_FRAME])
-            if comboBox:
-                return self._generateRoleName(comboBox, **args)
 
         if not force:
             doNotSpeak = [pyatspi.ROLE_FORM,
@@ -215,7 +188,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                           pyatspi.ROLE_MENU_ITEM,
                           pyatspi.ROLE_PARAGRAPH,
                           pyatspi.ROLE_SECTION,
-                          pyatspi.ROLE_TEXT,
                           pyatspi.ROLE_UNKNOWN]
         else:
             # We never ever want to speak 'unknown'
@@ -224,6 +196,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if not force and self._script.inDocumentContent(obj):
             doNotSpeak.append(pyatspi.ROLE_TABLE_CELL)
+            doNotSpeak.append(pyatspi.ROLE_TEXT)
             if args.get('formatType', 'unfocused') != 'basicWhereAmI':
                 doNotSpeak.append(pyatspi.ROLE_LIST_ITEM)
                 doNotSpeak.append(pyatspi.ROLE_LIST)
@@ -319,6 +292,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                                pyatspi.ROLE_TOOL_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_LIST_ITEM,
+                             pyatspi.ROLE_TABLE_ROW,
                              pyatspi.ROLE_TEXT]
 
         return speech_generator.SpeechGenerator._generateAncestors(
@@ -379,11 +353,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result.extend(speech_generator.SpeechGenerator.\
                                            generateSpeech(self, obj, **args))
             self._restoreRole(oldRole, args)
-        elif self._script.utilities.isEntry(obj):
-            oldRole = self._overrideRole(pyatspi.ROLE_ENTRY, args)
-            result.extend(speech_generator.SpeechGenerator.\
-                                           generateSpeech(self, obj, **args))
-            self._restoreRole(oldRole, args)
         elif self._script.utilities.isLink(obj):
             oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
             result.extend(speech_generator.SpeechGenerator.\
@@ -408,33 +377,40 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             self, obj, **args)
 
     def _generatePositionInList(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText') \
+           or not (_settingsManager.getSetting('enablePositionSpeaking') \
+                   or args.get('forceList', False)):
+            return []
+
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
+        menuRoles = [pyatspi.ROLE_MENU_ITEM,
+                     pyatspi.ROLE_CHECK_MENU_ITEM,
+                     pyatspi.ROLE_RADIO_MENU_ITEM,
+                     pyatspi.ROLE_MENU,
+                     pyatspi.ROLE_COMBO_BOX]
+        if obj.getRole() in menuRoles:
+            return super()._generatePositionInList(obj, **args)
+
         position = self.getAttribute(obj, "posinset")
         total = self.getAttribute(obj, "setsize")
-        if position and total and not obj.getRole() in \
-                [pyatspi.ROLE_MENU_ITEM,
-                 pyatspi.ROLE_TEAROFF_MENU_ITEM,
-                 pyatspi.ROLE_CHECK_MENU_ITEM,
-                 pyatspi.ROLE_RADIO_MENU_ITEM,
-                 pyatspi.ROLE_MENU]:
-            position = int(position)
-            total = int(total)
-            result = []
-            if (_settingsManager.getSetting('enablePositionSpeaking') \
-                or args.get('forceList', False)) \
-                and position >= 0:
-                result.append(self._script.formatting.getString(
-                    mode='speech',
-                    stringType='groupindex') \
-                    % {"index" : position,
-                    "total" : total})
-                result.extend(self.voice(speech_generator.SYSTEM))
-            return result
-        else:
-            return speech_generator.SpeechGenerator._generatePositionInList(
-                self, obj, **args)
+        if position is None or total is None:
+            return super()._generatePositionInList(obj, **args)
+
+        position = int(position)
+        total = int(total)
+        if position < 0 or total < 0:
+            return []
+
+        result = []
+        result.append(self._script.formatting.getString(
+            mode='speech',
+            stringType='groupindex') \
+            % {"index" : position,
+               "total" : total})
+        result.extend(self.voice(speech_generator.SYSTEM))
+        return result
 
     def _generateNewRadioButtonGroup(self, obj, **args):
         # TODO - JD: Looking at the default speech generator's method, this
@@ -467,8 +443,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not string:
             return []
 
-        if not self._script.utilities.isEntry(obj) \
-           and not self._script.utilities.isPasswordText(obj):
+        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
             string = string.strip()
 
         result = [string]
