@@ -142,6 +142,9 @@ class GeckoStructuralNavigation(structural_navigation.StructuralNavigation):
     def _presentLine(self, obj, offset):
         """Presents the first line of the object to the user."""
 
+        if self._presentWithSayAll(obj, offset):
+            return
+
         self._script.presentLine(obj, offset)
 
     def _presentObject(self, obj, offset):
@@ -150,6 +153,9 @@ class GeckoStructuralNavigation(structural_navigation.StructuralNavigation):
         if not obj:
             return
 
+        if self._presentWithSayAll(obj, offset):
+            return
+
         if obj.getRole() == pyatspi.ROLE_LINK:
             try:
                 obj.queryComponent().grabFocus()
@@ -251,10 +257,8 @@ class GeckoStructuralNavigation(structural_navigation.StructuralNavigation):
         """
 
         isMatch = False
-        if self._script.utilities.isEntry(obj) \
-           or self._script.utilities.isPasswordText(obj):
-            state = obj.getState()
-            isMatch = state.contains(pyatspi.STATE_FOCUSABLE) \
+        state = obj.getState()
+        isMatch = state.contains(pyatspi.STATE_FOCUSABLE) \
                   and state.contains(pyatspi.STATE_SENSITIVE) \
                   and state.contains(pyatspi.STATE_EDITABLE)
 
